// ShowImages.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <windows.h>
#include "gesture_render.h"
#include "util_pipeline.h"
#include "pxcgesture.h"

//     RGB 
#define COLOR_RED        RGB(255,0,0)
#define COLOR_GREEN      RGB(0,255,0)
#define COLOR_BLUE       RGB(0,0,255)
#define COLOR_YELLOW     RGB(255,255,0)
#define COLOR_PINK       RGB(255,0,255)

//         
static struct {
    PXCGesture::GeoNode::Label label;
    COLORREF color;
    int raduis;
} skeletons[]={
	        { PXCGesture::GeoNode::LABEL_BODY_HAND_LEFT|PXCGesture::GeoNode::LABEL_FINGER_THUMB, COLOR_GREEN, 2 },
            { PXCGesture::GeoNode::LABEL_BODY_HAND_LEFT|PXCGesture::GeoNode::LABEL_FINGER_INDEX, COLOR_RED, 2 },
            { PXCGesture::GeoNode::LABEL_BODY_HAND_LEFT|PXCGesture::GeoNode::LABEL_FINGER_MIDDLE, COLOR_BLUE, 2 },
            { PXCGesture::GeoNode::LABEL_BODY_HAND_LEFT|PXCGesture::GeoNode::LABEL_FINGER_RING, COLOR_YELLOW, 2 },
            { PXCGesture::GeoNode::LABEL_BODY_HAND_LEFT|PXCGesture::GeoNode::LABEL_FINGER_PINKY, COLOR_PINK, 2 }
};

//  
static void MapXY(float &x, float &y, PXCImage::ImageData *data, PXCImage::ImageInfo *depthInfo, PXCImage::ImageInfo *rgbInfo) {
    if (data->planes[2]) {
        if (x>=0 && y>=0 &&  x<depthInfo->width && y<depthInfo->height) {
          int index=((int)y)*depthInfo->width+x;
            int index2=2*index;
            x=((float*)data->planes[2])[index2]*rgbInfo->width;
            y=((float*)data->planes[2])[index2+1]*rgbInfo->height;
        }
    } else {
        if (depthInfo->width>0 && depthInfo->height>0) {
            x=x/depthInfo->width*rgbInfo->width;
            y=y/depthInfo->height*rgbInfo->height;
        }
    }
}

// 
bool GestureRender::RenderFrame(PXCImage *rgbImage, PXCGesture *detector, PXCGesture::Gesture *gdata, PXCImage *depthImage) {
  
    m_nodes.clear();
    if (detector) {
        PXCImage::ImageData depthData;
        memset(&depthData,0,sizeof(depthData));

        PXCImage::ImageInfo rgbInfo, depthInfo={0,0,0};
        rgbImage->QueryInfo(&rgbInfo);
        if (depthImage) {
            depthImage->QueryInfo(&depthInfo);
            pxcStatus sts=depthImage->AcquireAccess(PXCImage::ACCESS_READ,&depthData);
        }
		  for (int j=0;j<sizeof(skeletons)/sizeof(skeletons[0]);j++) {
              pxcUID user=0; 
			  PXCGesture::GeoNode data;
               if (detector->QueryNodeData(user,skeletons[j].label,&data)<PXC_STATUS_NO_ERROR) continue;

                float x=data.positionImage.x;
                float y=data.positionImage.y;

                MapXY(x,y,&depthData,&depthInfo,&rgbInfo);
                Node node={x,y,(data.radiusImage?data.radiusImage:skeletons[j].raduis), skeletons[j].color};
               m_nodes[std::pair<pxcUID,PXCGesture::GeoNode::Label>(user,skeletons[j].label)]=node;
            }
        }
	    return UtilRender::RenderFrame(rgbImage);
}

// -
void GestureRender::DrawMore(HDC hdc, double sx, double sy) {
	    for (std::map<std::pair<pxcUID,PXCGesture::GeoNode::Label>,Node>::iterator itrn=m_nodes.begin(); itrn!=m_nodes.end(); itrn++) {
        HPEN color=CreatePen(PS_SOLID, 4, itrn->second.color);
        SelectObject(hdc, color);

        int x=(int)(itrn->second.x*sx);
        int y=(int)(itrn->second.y*sy);
        int rx=(int)(itrn->second.radius*sx);
        int ry=(int)(itrn->second.radius*sy);
        MoveToEx(hdc, x, y, NULL);
        Arc(hdc,x-rx,y-ry,x+rx,y+ry,x+rx,y+ry,x+rx,y+ry);
        DeleteObject(color);
    }
		}

class GesturePipeline: public UtilPipeline 
{    

public:
    // 
    GesturePipeline (void):UtilPipeline(),m_render(L"Gesture Viewer") 
	{
         //    
         EnableGesture(); 
    }
	
	// OnGesture ,    
	virtual void PXCAPI OnGesture(PXCGesture::Gesture *data) 
    {
		PXCGesture* gest=QueryGesture();

		if (data->active) m_gdata = (*data);
		
    }
	  virtual bool OnNewFrame(void) 
    {
		return m_render.RenderFrame(QueryImage(PXCImage::IMAGE_TYPE_DEPTH),QueryGesture(), &m_gdata);

	}

protected:
    GestureRender m_render;
    PXCGesture::Gesture m_gdata;
};

int _tmain(int argc, _TCHAR* argv[])
{
	
	GesturePipeline pipeline;
    pipeline.LoopFrames();
    return 0;
}

